﻿using System;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Media;

using ChirpConnectNet;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 키
        /// </summary>
        private const string KEY = "[여기에 본인의 키를 설정해 주시기 바랍니다.]";

        /// <summary>
        /// 시크릿
        /// </summary>
        private const string SECRET = "[여기에 본인의 시크릿을 설정해 주시기 바랍니다.]";

        /// <summary>
        /// CHIRP 연결
        /// </summary>
        private ChirpConnect connect;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            #region 연결을 설정한다.

            try
            {
                this.connect = new ChirpConnect(KEY, SECRET, null);
            }
            catch(Exception exception)
            {
                this.startButton.IsEnabled        = false;
                this.sendButton.IsEnabled         = false;
                this.sendMessageTextBox.IsEnabled = false;

                ShowErrorMessage(exception.Message);

                return;
            }

            #endregion
            #region 이벤트를 설정한다.

            Closing                                  += Window_Closing;
            this.connect.OnStateChanged              += connect_OnStateChanged;
            this.connect.OnReceived                  += connect_OnReceived;
            this.connect.OnUnsupportedCaptureDevice  += connect_OnUnsupportedCaptureDevice;
            this.connect.OnUnsupportedRendererDevice += connect_OnUnsupportedRendererDevice;

            #endregion

            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                this.versionTextBlock.Text     = this.connect.Version;
                this.informationTextBlock.Text = this.connect.Info;

                this.stateTextBlock.Text = this.connect.State.ToString();
            }));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 닫을 경우 처리하기 - Window_Closing(sender, e)

        /// <summary>
        /// 윈도우 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            try
            {
                if(this.connect.State != SDKState.Stopped)
                {
                    this.connect.Stop();
                }

                this.connect.Dispose();
            }
            catch(Exception exception)
            {
                ShowErrorMessage(exception.Message);
            }
        }

        #endregion
        #region 연결 상태 변경시 처리하기 - connect_OnStateChanged(previousState, newState)

        /// <summary>
        /// 연결 상태 변경시 처리하기
        /// </summary>
        /// <param name="previousState">이전 상태</param>
        /// <param name="newState">신규 상태</param>
        private void connect_OnStateChanged(SDKState previousState, SDKState newState)
        {
            Brush color = Brushes.Blue;

            if(newState == SDKState.Sending || newState == SDKState.Receiving)
            {
                color = Brushes.LightGreen;
            }
            else if(newState == SDKState.Running)
            {
                color = Brushes.Black;
            }
            else if(newState == SDKState.Stopped)
            {
                color = Brushes.Red;
            }

            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                this.stateTextBlock.Foreground = color;
                this.stateTextBlock.Text       = newState.ToString();
            }));
        }

        #endregion
        #region 연결 수신시 처리하기 - connect_OnReceived(messageByteArray, channel)

        /// <summary>
        /// 연결 수신시 처리하기
        /// </summary>
        /// <param name="messageByteArray">메시지 바이트 배열</param>
        /// <param name="channel">채널</param>
        private void connect_OnReceived(byte[] messageByteArray, uint channel)
        {
            if(messageByteArray.Length > 0)
            {
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    string message = Encoding.UTF8.GetString(messageByteArray);

                    this.receiveMessageTextBox.Text = message;
                }));
            }
            else
            {
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    this.receiveMessageTextBox.Text = "메시지를 해석할 수 없습니다. 볼륨을 높여 주시기 바랍니다.";
                }));
            }
        }

        #endregion
        #region 연결 캡처 장치 미지원시 처리하기 - connect_OnUnsupportedCaptureDevice()

        /// <summary>
        /// 연결 캡처 장치 미지원시 처리하기
        /// </summary>
        private void connect_OnUnsupportedCaptureDevice()
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                MessageBox.Show
                (
                    this,
                    "선택된 캡처 장치가 지원되지 않습니다. 다른 장치를 선택해 주시기 바랍니다.",
                    "확인",
                    MessageBoxButton.OK,
                    MessageBoxImage.Information
                );
            }));
        }

        #endregion
        #region 연결 렌더러 장치 미지원시 처리하기 - connect_OnUnsupportedRendererDevice()

        /// <summary>
        /// 연결 렌더러 장치 미지원시 처리하기
        /// </summary>
        private void connect_OnUnsupportedRendererDevice()
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                MessageBox.Show
                (
                    this,
                    "선택된 렌더러 장치가 지원되지 않습니다. 다른 장치를 선택해 주시기 바랍니다.",
                    "확인",
                    MessageBoxButton.OK,
                    MessageBoxImage.Information
                );
            }));
        }

        #endregion
        #region 시작하기 버튼 클릭시 처리하기 - startButton_Click(sender, e)

        /// <summary>
        /// 시작하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void startButton_Click(object sender, RoutedEventArgs e)
        {
            if(this.startButton.Content.ToString() == "시작하기")
            {
                ChirpError chirpError = this.connect.Start(AudioMode.PlayingListening);

                if(chirpError != null)
                {
                    ShowErrorMessage("시작시 에러 : " + chirpError.ToString());

                    return;
                }

                this.sendButton.IsEnabled = true;

                this.startButton.Content = "중단하기";
            }
            else if(this.startButton.Content.ToString() == "중단하기")
            {
                ChirpError chirpError = this.connect.Stop();

                if(chirpError != null)
                {
                    ShowErrorMessage("중단시 에러 : " + chirpError);

                    return;
                }

                this.sendButton.IsEnabled = false;

                this.startButton.Content = "시작하기";
            }
        }

        #endregion
        #region 송신하기 버튼 클릭시 처리하기 - sendButton_Click(sender, e)

        /// <summary>
        /// 송신하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sendButton_Click(object sender, RoutedEventArgs e)
        {
            if(string.IsNullOrEmpty(this.sendMessageTextBox.Text))
            {
                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    MessageBox.Show
                    (
                        this,
                        "송신 메시지 항목을 입력해 주시기 바랍니다.",
                        "확인",
                        MessageBoxButton.OK,
                        MessageBoxImage.Information
                    );
                }));
            }
            else
            {
                string message = this.sendMessageTextBox.Text;

                byte[] byteArray = Encoding.UTF8.GetBytes(message);

                ChirpError chirpError = this.connect.Send(byteArray);

                if(chirpError != null)
                {
                    ShowErrorMessage("송신시 에러 : " + chirpError.ToString());
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 에러 메시지 보여주기 - ShowErrorMessage(message)

        /// <summary>
        /// 에러 메시지 보여주기
        /// </summary>
        /// <param name="message">메시지</param>
        private void ShowErrorMessage(string message)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                this.stateTextBlock.Foreground = Brushes.Red;
                this.stateTextBlock.Text       = message;
            }));
        }

        #endregion
    }
}